import sys
import os
from os.path import basename, join, isdir
import argparse
import time
import tigerseg.segment
import tigerseg.methods.mprage
import glob
import platform
import nibabel as nib



def main():

    default_model = 'mprage_v0004_bet_full.onnx'    
    parser = argparse.ArgumentParser()
    parser.add_argument('input',  type=str, nargs='+', help='Path to the input image, can be a folder for the specific format(nii.gz)')
    parser.add_argument('-o', '--output', default=None, help='File path for output segmentation, default: the directory of input files')
    parser.add_argument('-g', '--gpu', action='store_true', help='Using GPU')
    parser.add_argument('-m', '--betmask', action='store_true', help='Producing bet mask')
    parser.add_argument('-a', '--aseg', action='store_true', help='Producing aseg mask')
    parser.add_argument('-b', '--bet', action='store_true', help='Producing bet mask')
    parser.add_argument('-d', '--deepgm', action='store_true',
                        help='Producing deepgm mask')
    parser.add_argument('-f', '--fast', action='store_true', help='Fast processing with low-resolution model')
    parser.add_argument('--model', default=default_model, type=str, help='Specifies the modelname')
    #parser.add_argument('--report',default='True',type = strtobool, help='Produce additional reports')
    args = parser.parse_args()

    input_file_list = args.input
    if os.path.isdir(args.input[0]):
        input_file_list = glob.glob(join(args.input[0], '*.nii'))
        input_file_list += glob.glob(join(args.input[0], '*.nii.gz'))

    elif '*' in args.input[0]:
        input_file_list = glob.glob(args.input[0])

    output_dir = args.output

    #model_name = args.model
    #model_aseg = 'mprage_v0006_aseg43_full.onnx'

    
    if args.fast:
        model_name = 'mprage_v0002_bet_kuor128.onnx'
        model_aseg = 'mprage_v0001_aseg43_MXRWr128.onnx'
    else:
        model_name = args.model
        model_aseg = 'mprage_v0005_aseg43_full.onnx'
    

    print('Total nii files:', len(input_file_list))

    for f in input_file_list:

        print('Processing :', os.path.basename(f))
        t = time.time()
            
        input_data = tigerseg.methods.mprage.read_file(model_name, f)

        mask = tigerseg.segment.apply(model_name, input_data,  GPU=args.gpu)

        f_output_dir = output_dir

        if f_output_dir is None:
            f_output_dir = os.path.dirname(os.path.abspath(f))
        else:
            os.makedirs(f_output_dir, exist_ok=True)

        mask_file, mask_niimem = tigerseg.methods.mprage.write_file(model_name,
                                            f, f_output_dir, 
                                            mask, postfix='tbetmask', inmem=True)


        
        if args.bet:
            input_nib = nib.load(f)
            bet = input_nib.get_fdata() * mask_niimem.get_fdata()
            bet = bet.astype(
                input_nib.dataobj.dtype)

            bet = nib.Nifti1Image(bet, input_nib.affine, input_nib.header)

            output_file = basename(f).replace(
                '.nii', f'_tbet.nii')
            output_file = join(f_output_dir, output_file)
            nib.save(bet, output_file)
            print('Writing output file: ', output_file)
            
             
        if args.betmask:
            nib.save(mask_niimem, mask_file)
            print('Writing output file: ', mask_file)

        if args.aseg or args.deepgm:
            input_nib = nib.load(f)
            input_data = tigerseg.methods.mprage.read_file(model_aseg, f)
            asegmask = tigerseg.segment.apply(
                model_aseg, input_data,  GPU=args.gpu)
            aseg_file, aseg_niimem = tigerseg.methods.mprage.write_file(model_aseg,
                                                                        f, f_output_dir,
                                                                        asegmask, postfix='aseg', inmem=True)
            aseg = aseg_niimem.get_fdata() * mask_niimem.get_fdata()
            aseg = aseg.astype(int)

            if args.aseg:
                asegnii = nib.Nifti1Image(aseg, input_nib.affine, input_nib.header)
                nib.save(asegnii, aseg_file)
                print('Writing output file: ', aseg_file)

            if args.deepgm:
                deepgm = aseg * 0
                count = 0
                for ii in [10, 49, 11, 50, 12, 51, 13, 52, 17, 53, 18, 54]:
                    count += 1
                    deepgm[aseg==ii] = count

                deepgm = deepgm.astype(int)

                output_file = basename(f).replace(
                    '.nii', f'_deegm.nii')
                output_file = join(f_output_dir, output_file)
                deepgm = nib.Nifti1Image(
                    deepgm, input_nib.affine, input_nib.header)
                nib.save(deepgm, output_file)
                print('Writing output file: ', output_file)

            
            
            





        print('Processing time: %d seconds' %  (time.time() - t))




if __name__ == "__main__":
    main()
    if platform.system() == 'Windows':
        os.system('pause')



import sys
import os
import argparse
from .. import segment
from distutils.util import strtobool
import glob
from scipy.io import savemat
import nibabel as nib
import numpy as np


def get_report(input_file, output_file):
    
    temp = nib.load(output_file)
    mask4d = temp.get_fdata()
    voxel_size = temp.header.get_zooms()
    
    LV_vol = np.sum(mask4d==1, axis=(0, 1, 2))* np.prod(voxel_size[0:3]) / 1000.
    LVM_vol = np.sum(mask4d==2, axis=(0, 1, 2))* np.prod(voxel_size[0:3]) / 1000.
    RV_vol = np.sum(mask4d==3, axis=(0, 1, 2))* np.prod(voxel_size[0:3]) / 1000.
    
    dict1 = {"input":np.asanyarray(nib.load(input_file).dataobj),
             'LV': (mask4d==1)*1,
             'LVM':(mask4d==2)*1,
             'RV': (mask4d==3)*1,
             'LV_vol': LV_vol,
             'LVM_vol': LVM_vol,
             'RV_vol': RV_vol}

    savemat(output_file.replace('.nii.gz', '.mat'), dict1, do_compression=True)


def path(string):
    if os.path.exists(string):
        return string
    else:
        sys.exit(f'File not found: {string}')


def main():
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', metavar='INPUT_FILE', required=True, type=str, help='Path to the input image, can be a folder for the specific format(nii.gz)', nargs='+')
    parser.add_argument('-o', '--output', metavar='OUTPUT_DIR', default=None, type=path, help='Filepath for output segmentation, default: the directory of input files')
    parser.add_argument('--model', default='cine4d_v0002_xyz_mms12acdc', type=str, help='specifies the modelname')
    parser.add_argument('--GPU',default='False',type = strtobool, help='True: GPU, False: CPU, default: False, CPU')
    parser.add_argument('--report',default='True',type = strtobool, help='Produce additional reports')
    args = parser.parse_args()
    
    input_file_list = []
    for arg in args.input:
        input_file_list += glob.glob(arg)

    output_file_list = segment.apply_files(model_name=args.model,
                        input_file_list=input_file_list,
                        output_dir=args.output,                        
                        GPU=args.GPU)

    if args.report:
        for ii in range(len(input_file_list)):
            input_file = input_file_list[ii]
            output_file = output_file_list[ii]
            get_report(input_file, output_file)



if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from math import factorial
import os
import warnings
import urllib.request
from os.path import join, isdir, basename, isfile, dirname
import time
import importlib
import nibabel as nib
import numpy as np


warnings.filterwarnings("ignore", category=UserWarning)
nib.Nifti1Header.quaternion_threshold = -100

#model_server = 'https://github.com/htylab/tigerseg/releases/download/modelhub/'
model_server = 'https://data.mrilab.org/onnxmodel/release016'
model_path = join(os.path.dirname(os.path.abspath(__file__)), 'models')
# print(model_path)
os.makedirs(model_path, exist_ok=True)

def apply_files(model_name, input_file_list, output_dir=None, GPU=False, model_path=model_path):

    seg_method = basename(model_name).split('_')[0]
    seg_module = importlib.import_module('tigerseg.methods.' + seg_method)
    
    print('Total nii files:', len(input_file_list))

    output_file_list = []

    for f in input_file_list:

        print('Predicting:', f)
        t = time.time()
          
        input_data = seg_module.read_file(model_name, f)
        
        mask = apply(model_name, input_data, GPU=GPU, model_path=model_path)

        if output_dir is not None:
            output_file, _ = seg_module.write_file(model_name, f, output_dir, mask)
            output_file_list.append(output_file)

        else:
            output_file, _ = seg_module.write_file(model_name, f, dirname(os.path.abspath(f)), mask)
            output_file_list.append(output_file)


        print('Processing time: %d seconds' %  (time.time() - t))


    return output_file_list


def apply(model_name, input_data, GPU=False, model_path=model_path):

    #download model files
    model_ffs = []
    for f in model_name.replace('@', '#').replace('*', '#').split('#'):

        if isfile(f):
            model_ffs.append(f)
            continue
        if '.onnx' in f:
            fn = f
        else:
            fn = f + '.onnx'

        model_url = model_server + fn
        model_file = join(model_path, fn)
        model_ffs.append(model_file)
        if not os.path.exists(model_file):
            print(f'Downloading model files....')
            print(model_url, model_file)
            urllib.request.urlretrieve(model_url, model_file)
        
    #todo: verify model files

        #apply 只處理 numpy array --> model --> mask output
    #有三種模式
    #

    seg_method = basename(model_ffs[0]).split('_')[0]
    seg_module = importlib.import_module('tigerseg.methods.' + seg_method)
    run_SingleModel = getattr(seg_module, 'run_SingleModel')

    
    if '#' in model_name: #model ensemble by softmax summation
        mask_softmax_sum = 0

        for model_ff in model_ffs:
            mask_pred, mask_softmax = run_SingleModel(model_ff, input_data, GPU)
            mask_softmax_sum += mask_softmax

        mask_pred = np.argmax(mask_softmax_sum, axis=0)

    elif '*' in model_name: #mask segmentation
        print('Intersection of masks')

        mask1, mask_softmax = run_SingleModel(model_ffs[0], input_data, GPU)
        mask2, mask_softmax = run_SingleModel(model_ffs[1], input_data, GPU)

        mask_pred = mask1 * (mask2 > 0)
        

    elif '@' in model_name: #todo two-stage segmenation
        print('Two-stage model')
        model1_ff, model2_ff = model_ffs[0], model_ffs[1]
        mask1, mask_softmax = run_SingleModel(model1_ff, input_data, GPU)
        mask2, mask_softmax = run_SingleModel(model2_ff, input_data, GPU)

        mask_pred = mask1 * mask2
        

    else: #single model mode
        model_ff = model_ffs[0]
        mask_pred, mask_softmax = run_SingleModel(model_ff, input_data, GPU)
    
    if 'post' in dir(seg_module):
        mask_pred = seg_module.post(mask_pred)

    return mask_pred





import glob
from os.path import join, basename, isdir
import os
import numpy as np
import nibabel as nib
import onnxruntime as ort
from scipy.special import softmax
nib.Nifti1Header.quaternion_threshold = -100


def run_SingleModel(model_ff, input_data, GPU):

 

    so = ort.SessionOptions()
    so.intra_op_num_threads = 4
    so.inter_op_num_threads = 4

    '''
    
    try:
        if os.cpu_count() is None:
            so.intra_op_num_threads = 1
            so.inter_op_num_threads = 1
        else:
            so.intra_op_num_threads = os.cpu_count()
            so.inter_op_num_threads = os.cpu_count()
    except:
        so.intra_op_num_threads = 1
        so.inter_op_num_threads = 1
    
    so.intra_op_num_threads = 4
    so.inter_op_num_threads = 4
    print('************', so.intra_op_num_threads)
    '''
    if GPU and (ort.get_device() == "GPU"):
        #ort.InferenceSession(model_file, providers=['CPUExecutionProvider'])
        session = ort.InferenceSession(model_ff,
                                       providers=['CUDAExecutionProvider'],
                                       sess_options=so)
    else:
        session = ort.InferenceSession(model_ff,
                                       providers=['CPUExecutionProvider'],
                                       sess_options=so)


    xyzt_mode=basename(model_ff).split('_')[2]
    

    data = input_data.copy()
    xx, yy, zz, tt = data.shape

    if xyzt_mode == 'xyt':
        data = np.transpose(data, [0, 1, 3, 2])

    if xyzt_mode == 'xy':
        xx, yy, zz, tt = data.shape
        data = np.reshape(data, [xx, yy, zz*tt])

    #affine = temp.affine
    #zoom = temp.header.get_zooms()

    
    mask_pred4d = data * 0
    mask_softmax4d = np.zeros(np.insert(data.shape, 0, 4))

    
    for tti in range(data.shape[-1]):

        image_raw = data[..., tti]
        image = image_raw[None, ...][None, ...]
        if np.max(image) == 0:
            continue
        image = image/np.max(image)

        logits = session.run(None, {"modelInput": image.astype(np.float32)})[0]

        mask_pred = post(np.argmax(logits[0, ...], axis=0))
        mask_softmax = softmax(logits[0, ...], axis=0)

        #print(xyzt_mode, tti, image.max(), mask_pred.max(), image.shape)

        mask_pred4d[..., tti] = mask_pred
        mask_softmax4d[..., tti] = mask_softmax


    if xyzt_mode == 'xyt':
        mask_pred4d = np.transpose(mask_pred4d, [0, 1, 3, 2])
        mask_softmax4d = np.transpose(mask_softmax4d, [0, 1, 2, 4, 3])

    if xyzt_mode == 'xy':
        mask_pred4d = np.reshape(mask_pred4d, [xx, yy, zz, tt])
        mask_softmax4d = np.reshape(mask_softmax4d, [4, xx, yy, zz, tt])

    return mask_pred4d, mask_softmax4d


def read_file(model_ff, input_file):

    return nib.load(input_file).get_fdata()

def write_file(model_ff, input_file, output_dir, mask, inmem=False):

    if not isdir(output_dir):
        print('Output dir does not exist.')
        return 0

    output_file = basename(input_file).replace('.nii.gz', '').replace('.nii', '') 
    output_file = output_file + '_pred.nii.gz'
    output_file = join(output_dir, output_file)
    print('Writing output file: ', output_file)

    input_nib = nib.load(input_file)
    affine = input_nib.affine
    zoom = input_nib.header.get_zooms()   
    result = nib.Nifti1Image(mask.astype(np.uint8), affine)
    result.header.set_zooms(zoom)

    #if 'mprage' in model_name:
    #result = resample_to_img(result, f, interpolation="nearest")
    if not inmem:
        nib.save(result, output_file)

    return output_file, result



def post(mask):

    def getLarea(input_mask):
        from scipy import ndimage
        try:
            labeled_mask, cc_num = ndimage.label(input_mask)
            mask = (labeled_mask == (np.bincount(labeled_mask.flat)[1:].argmax() + 1))
        except:
            mask = input_mask
        return mask

    masknew = mask * 0
    for jj in range(1, int(mask.max()) + 1):
        masknew[getLarea(mask == jj)] = jj

    return masknew



import glob
from os.path import join, basename, isdir
import os
import numpy as np
from scipy.io import savemat
import nibabel as nib
import onnxruntime as ort
from scipy.special import softmax
from skimage import transform
from nilearn.image import reorder_img, resample_to_img, resample_img

labels = (2,3,4,5,7,8,10,11,12,13,14,15,16,17,18,24,26,28,30,31,41,42,43,44,46,47,49,50,51,52,53,54,58,60,62,63,77,85,251,252,253,254,255)
nib.Nifti1Header.quaternion_threshold = -100


def getLarea(input_mask):
    from scipy import ndimage
    labeled_mask, cc_num = ndimage.label(input_mask)
    if cc_num > 0:
        labeled_mask, cc_num = ndimage.label(input_mask)
        mask = (labeled_mask == (np.bincount(
            labeled_mask.flat)[1:].argmax() + 1))
    else:
        mask = input_mask
    return mask

def get_affine(mat_size=256):

    target_shape = np.array((mat_size, mat_size, mat_size))
    new_resolution = [256/mat_size, ]*3
    new_affine = np.zeros((4, 4))
    new_affine[:3, :3] = np.diag(new_resolution)
    # putting point 0,0,0 in the middle of the new volume - this could be refined in the future
    new_affine[:3, 3] = target_shape * new_resolution/2.*-1
    new_affine[3, 3] = 1.
    #print(model_ff, target_shape)
    return new_affine, target_shape
'''
def get_resample(input_file, matrix=128):

    target_affine = [  [  1,  0,  0,  -94], [  0,  1,  0, -111],
                [  0,  0,  1, -147], [  0,  0,  0,    1]]
    
    target_affine = np.array(target_affine)
    target_shape = [192, 256, 256]

    return target_affine, target_shape
'''
def run_SingleModel(model_ff, input_data, GPU):

    seg_mode, model_str = basename(model_ff).split('_')[2:4] #aseg43, bet  

    data = input_data.copy()
    do_resize = False

    if 'r128' in model_str and data.shape != (128, 128, 128):

        data = transform.resize(data, (128, 128, 128),
                                preserve_range=True)
        do_resize = True
    elif 'r256' in model_str and data.shape != (256, 256, 256):

        data = transform.resize(data, (256, 256, 256),
                                preserve_range=True)
        do_resize = True

    image = data[None, ...][None, ...]
    image = image/np.max(image)

    logits = predict(model_ff, image, GPU)[0, ...]

    label_num = dict()
    label_num['bet'] = 2
    label_num['aseg43'] = 44

    if label_num[seg_mode] > logits.shape[0]:
        #sigmoid mode
        th = 0.5
        if seg_mode == 'bet':
            from scipy.special import expit
            logits = expit(logits)
            th = 0.5
        mask_pred = np.argmax(logits, axis=0) + 1
        mask_pred[np.max(logits, axis=0) <= th] = 0
        if seg_mode == 'bet':
            mask_pred = getLarea(mask_pred)
        prob = logits
    else:
        #softmax mode
        mask_pred = np.argmax(logits, axis=0)
        prob = softmax(logits, axis=0)

    if seg_mode == 'aseg43':

        mask_pred_relabel = mask_pred * 0
        for ii in range(len(labels)):
            mask_pred_relabel[mask_pred == (ii + 1)] = labels[ii]

        mask_pred = mask_pred_relabel

    if do_resize:
        mask_pred = transform.resize(mask_pred, input_data.shape,
                                    order=0, preserve_range=True)

   
    return mask_pred.astype(np.uint8), prob


def read_file(model_ff, input_file):

    seg_mode, model_str = basename(model_ff).split('_')[2:4] #aseg43, bet  

    if 'r128' in model_str:

        affine, shape = get_affine(mat_size=128)
        vol = resample_img(nib.load(input_file),
                           target_affine=affine, target_shape=shape).get_fdata()

    elif 'r256' in model_str:

        affine, shape = get_affine(mat_size=256)
        vol = resample_img(nib.load(input_file),
                           target_affine=affine, target_shape=shape).get_fdata()
    else:
        #vol = nib.load(input_file).get_fdata()

        vol = reorder_img(nib.load(input_file), resample='linear').get_fdata()
        #print(reorder_img(nib.load(input_file), resample='linear').affine)

    #print(vol.shape)
    return vol 


def write_file(model_ff, input_file, output_dir,
               mask, postfix=None, dtype='mask', inmem=False):

    if not isdir(output_dir):
        print('Output dir does not exist.')
        return 0
    seg_mode, model_str = basename(model_ff).split('_')[2:4] #aseg43, bet 
    if postfix is None:
        postfix = seg_mode
    
    output_file = basename(input_file).replace('.nii', f'_{postfix}.nii')    

    output_file = join(output_dir, output_file)
    
    input_nib = nib.load(input_file)
    input_affine = input_nib.affine
    zoom = input_nib.header.get_zooms()    

    if 'r128' in model_str:
        target_affine, _ = get_affine(mat_size=128)
    elif 'r256' in model_str:
        target_affine, _ = get_affine(mat_size=256)         
    else:
        target_affine = reorder_img(nib.load(input_file), resample='linear').affine

    if dtype == 'orig':
        result = nib.Nifti1Image(mask.astype(input_nib.dataobj.dtype), target_affine)
    else:
        result = nib.Nifti1Image(mask.astype(np.uint8), target_affine)
    result = resample_to_img(result, input_nib, interpolation="nearest")
    result.header.set_zooms(zoom)

    if not inmem:        
        nib.save(result, output_file)
        print('Writing output file: ', output_file)

    return output_file, result


def predict(model, data, GPU):

    so = ort.SessionOptions()
    so.intra_op_num_threads = 4
    so.inter_op_num_threads = 4

    if GPU and (ort.get_device() == "GPU"):
        #ort.InferenceSession(model_file, providers=['CPUExecutionProvider'])
        session = ort.InferenceSession(model,
                                       providers=['CUDAExecutionProvider'],
                                       sess_options=so)
    else:
        session = ort.InferenceSession(model,
                                       providers=['CPUExecutionProvider'],
                                       sess_options=so)
    data_type = 'float64'
    if session.get_inputs()[0].type == 'tensor(float)':
        data_type = 'float32'  

    return session.run(None, {session.get_inputs()[0].name: data.astype(data_type)}, )[0]







import sys
import os
import argparse
from .. import segment
from distutils.util import strtobool
import glob

def path(string):
    if os.path.exists(string):
        return string
    else:
        sys.exit(f'File not found: {string}')


def main():


    #default_model = 'mprage_v0001_aseg43_MXRW*mprage_v0002_bet_kuo' #weng's thesis
    #default_model = 'mprage_v0003_aseg43_WangM1r256*mprage_v0004_bet_full'  # wang's thesis
    default_model = 'mprage_v0006_aseg43_full*mprage_v0004_bet_full'  # hty version
    

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', metavar='INPUT_FILE', required=True, type=str, nargs='+', help='Path to the input image, can be a folder for the specific format(nii.gz)')
    parser.add_argument('-o', '--output', metavar='OUTPUT_DIR', default=None, type=path, help='Filepath for output segmentation, default: the directory of input files')
    parser.add_argument('--model', default=default_model, type=str, help='specifies the modelname')
    parser.add_argument('--GPU',default='False',type=strtobool, help='True: GPU, False: CPU, default: False, CPU')
    parser.add_argument('--report',default='True',type=strtobool, help='Produce additional reports')
    args = parser.parse_args()
    
    input_file_list = []
    for arg in args.input:
        input_file_list += glob.glob(arg)

    segment.apply_files(model_name=args.model,
                        input_file_list=input_file_list,
                        output_dir=args.output,                        
                        GPU=args.GPU)



if __name__ == "__main__":
    main()



from scipy.ndimage import gaussian_filter
from scipy.signal import medfilt2d
from os.path import basename, join
import SimpleITK as sitk
from os.path import join, basename, isdir
import numpy as np
import nibabel as nib
import onnxruntime as ort
from scipy.special import softmax
import tqdm


nib.Nifti1Header.quaternion_threshold = -100

def run_SingleModel(model_ff, input_data, GPU):

    so = ort.SessionOptions()
    so.intra_op_num_threads = 4
    so.inter_op_num_threads = 4

    if GPU and (ort.get_device() == "GPU"):
        #ort.InferenceSession(model_file, providers=['CPUExecutionProvider'])
        session = ort.InferenceSession(model_ff,
                                       providers=['CUDAExecutionProvider'],
                                       sess_options=so)
    else:
        session = ort.InferenceSession(model_ff,
                                       providers=['CPUExecutionProvider'],
                                       sess_options=so)


    seg_mode = basename(model_ff).split('_')[2] #3dunet

    orig_data = input_data.copy()   

    if len(orig_data.shape) == 3:
        
        output_vol, vdm_pred = correct_3dvol(session, orig_data)
    
    elif len(orig_data.shape) == 4:
        output_vol = orig_data * 0
        vdm_pred = orig_data * 0.0
        for nn in tqdm.tqdm(range(orig_data.shape[-1])):
            orig_data3d = orig_data[..., nn]
            if nn == 0:
                output_vol[..., nn], vdm_pred = correct_3dvol(
                session, orig_data3d)
            else:
                output_vol[..., nn] = apply_vdm_3d(orig_data3d, vdm_pred)



    return output_vol.astype(np.int16), vdm_pred



    return output_vol, mask_softmax


def read_file(model_ff, input_file):
    #reorder_img : reorient image to RAS 

    return nib.load(input_file).get_fdata()


def write_file(model_ff, input_file, output_dir, vol_out, inmem=False):

    if not isdir(output_dir):
        print('Output dir does not exist.')
        return 0

    output_file = basename(input_file).replace('.nii.gz', '').replace('.nii', '') 
    output_file = output_file + '_vdm.nii.gz'
    output_file = join(output_dir, output_file)
    print('Writing output file: ', output_file)

    input_nib = nib.load(input_file)
    affine = input_nib.affine
    zoom = input_nib.header.get_zooms()
    result = nib.Nifti1Image(vol_out.astype(np.int16), affine)

    result.header.set_zooms(zoom)

    if not inmem:
        nib.save(result, output_file)

    return output_file, result


def predict(model, data):
    if model.get_inputs()[0].type == 'tensor(float)':
        return model.run(None, {model.get_inputs()[0].name: data.astype('float32')}, )[0]
    else:
        return model.run(None, {model.get_inputs()[0].name: data.astype('float64')}, )[0]


def apply_vdm_2d(ima, vdm, readout=1):

    arr = np.stack([vdm*readout, vdm*0], axis=-1)
    displacement_image = sitk.GetImageFromArray(arr, isVector=True)

    jac = sitk.DisplacementFieldJacobianDeterminant(displacement_image)
    tx = sitk.DisplacementFieldTransform(displacement_image)
    ref = sitk.GetImageFromArray(ima*0)
    resampler = sitk.ResampleImageFilter()
    resampler.SetReferenceImage(ref)
    # sitkNearestNeighbor, sitk.sitkLinear
    resampler.SetInterpolator(sitk.sitkLinear)
    resampler.SetTransform(tx)

    new_ima = resampler.Execute(sitk.GetImageFromArray(ima))
    new_ima = sitk.GetArrayFromImage(new_ima)
    jac_np = sitk.GetArrayFromImage(jac)
    return new_ima*jac_np


def apply_vdm_3d(vol, vdm_pred):
    output_vol = vol * 0

    for nslice in range(vol.shape[-1]):
        ima_org = vol[..., nslice].astype(np.float)
        vdm_slice = vdm_pred[..., nslice]
        ima_transform = apply_vdm_2d(ima_org, vdm_slice)

        output_vol[..., nslice] = ima_transform

    return output_vol


def correct_3dvol(session, orig_data):
    image = orig_data[None, ...][None, ...]

    image = image/np.max(image)

    #sigmoid = session.run(None, {"modelInput": image.astype(np.float64)})[0]

    logits = predict(session, image)

    mask_softmax = softmax(logits[0, ...], axis=0)

    softmax_all = 0
    for ii in range(101):
        softmax_all += mask_softmax[ii, ...] * ii

    vdm_pred = gaussian_filter(softmax_all*0.4 - 20, 0.5).astype(np.float)

    output_vol = apply_vdm_3d(orig_data, vdm_pred)

    return output_vol, vdm_pred



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

classifiers = [
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: MIT License'
]

setup(
     name='tigerseg',
     version='0.1.7',
     description='Processing MRI images based on deep-learning',
     long_description_content_type='text/x-rst',
     url='https://github.com/htylab',
     author='Biomedical Imaging Lab, Taiwan Tech',
     author_email='',
     License='MIT',
     classifiers=classifiers,
     keywords='MRI segmentation',
     package_dir={"": "src"},
     packages=find_packages(where="src"),
     entry_points={
        'console_scripts': [
            'tigerseg = tigerseg.console.__main__:main',
            'cine4d = tigerseg.console.__cine4d__:main',
            'aseg = tigerseg.console.__aseg__:main',
            'aseg2 = tigerseg.console.__aseg2__:main',
            'pybet = tigerseg.console.__tigerbet__:main',
            'vdm = tigerseg.console.__vdm__:main',
        ]
    },
     python_requires='>=3.6',
     install_requires=[
             'numpy>=1.16.0',
             'nilearn>=0.9.1',
             'onnxruntime>1.9.0',
             'simpleitk>=2.0.0',
             'scikit-image',
             'tqdm'
         ]
)
